/*global module Float32Array Float64Array*/
(function (){"use strict";
	function Quaternion (quat) {return this.initQuaternion(quat);}
	var quaternion = Quaternion.prototype;
	Quaternion.check = function (obj) {
		return obj && obj.initQuaternion === quaternion.initQuaternion;
	};
	Quaternion.create = function (obj) {
		obj = obj || [];
		obj.length = 4;
		obj.initQuaternion = quaternion.initQuaternion || function (quat) {
			quat = quat || [];
			this[0] = quat[0] || 0;
			this[1] = quat[1] || 0;
			this[2] = quat[2] || 0;
			this[3] = quat[3] || 0;
			return this;
		};
		obj.set = function (quat) {
			return this.initQuaternion(quat);
		};
		obj.copy = function () {
			return new Quaternion(this);
		};
		obj.calculateW = function () {
			var x = this[0], y = this[1], z = this[2];
			this[3] = -Math.sqrt(Math.abs(1.0 - x*x - y*y - z*z));
			return this;
		};
		obj.inverse = function () {
			this[0] *= -1;
			this[1] *= -1;
			this[2] *= -1;
			return this;
		};
		obj.length = function () {
			var x = this[0], y = this[1], z = this[2], w = this[3];
			return Math.sqrt(x*x + y*y + z*z + w*w);
		};
		obj.normalize = function () {
			var x = this[0], y = this[1], z = this[2], w = this[3],
			    len = Math.sqrt(x*x + y*y + z*z + w*w);
			if(len === 0) {
				this[0] = this[1] = this[2] = this[3] = 0;
			} else {
				len = 1/len;
				this[0] = x * len;
				this[1] = y * len;
				this[2] = z * len;
				this[3] = w * len;
			}
			return this;
		};
		obj.multiply = function (quat) {
			var qax = this[0], qay = this[1], qaz = this[2], qaw = this[3],
			    qbx = quat[0], qby = quat[1], qbz = quat[2], qbw = quat[3];
			this[0] = qax*qbw + qaw*qbx + qay*qbz - qaz*qby;
			this[1] = qay*qbw + qaw*qby + qaz*qbx - qax*qbz;
			this[2] = qaz*qbw + qaw*qbz + qax*qby - qay*qbx;
			this[3] = qaw*qbw - qax*qbx - qay*qby - qaz*qbz;
			return this;
		};
		obj.multiplyVec3 = function (vec) {
			var  x =  vec[0],  y =  vec[1],  z =  vec[2],
			    qx = this[0], qy = this[1], qz = this[2], qw = this[3],
			    ix = qw*x + qy*z - qz*y,
			    iy = qw*y + qz*x - qx*z,
			    iz = qw*z + qx*y - qy*x,
			    iw = -qx*x - qy*y - qz*z;
			vec[0] = ix*qw + iw*-qx + iy*-qz - iz*-qy;
			vec[1] = iy*qw + iw*-qy + iz*-qx - ix*-qz;
			vec[2] = iz*qw + iw*-qz + ix*-qy - iy*-qx;
			return this;
		};
		obj.toMat3 = function () {
			var x = this[0], y = this[1], z = this[2], w = this[3],
			    x2 = x + x, y2 = y + y, z2 = z + z,
			    xx = x*x2, xy = x*y2, xz = x*z2,
			    yy = y*y2, yz = y*z2, zz = z*z2,
			    wx = w*x2, wy = w*y2, wz = w*z2;
			return [
				1 - (yy + zz),
				xy - wz,
				xz + wy,
				xy + wz,
				1 - (xx + zz),
				yz - wx,
				xz - wy,
				yz + wx,
				1 - (xx + yy)
			];
		};
		obj.toMat4 = function (quat) {
			var x = this[0], y = this[1], z = this[2], w = this[3],
			    x2 = x + x, y2 = y + y, z2 = z + z,
			    xx = x*x2, xy = x*y2, xz = x*z2,
			    yy = y*y2, yz = y*z2, zz = z*z2,
			    wx = w*x2, wy = w*y2, wz = w*z2;
			return [
				1 - (yy + zz),
				xy - wz,
				xz + wy,
				0,
				xy + wz,
				1 - (xx + zz),
				yz - wx,
				0,
				xz - wy,
				yz + wx,
				1 - (xx + yy),
				0,
				0,
				0,
				0,
				1
			];
		};
		obj.slerp = function (quat, slerp) {
			var cosHalfTheta = 
				this[0]*this[0] + this[1]*this[1] + this[2]*this[2] + this[3]*this[3];
			if (Math.abs(cosHalfTheta) >= 1.0){
				return this;
			}
			var halfTheta = Math.acos(cosHalfTheta),
			    sinHalfTheta = Math.sqrt(1.0 - cosHalfTheta*cosHalfTheta);
			if (Math.abs(sinHalfTheta) < 0.001){
				this[0] = (this[0]*0.5 + quat[0]*0.5);
				this[1] = (this[1]*0.5 + quat[1]*0.5);
				this[2] = (this[2]*0.5 + quat[2]*0.5);
				this[3] = (this[3]*0.5 + quat[3]*0.5);
				return this;
			}
			var ratioA = Math.sin((1 - slerp)*halfTheta) / sinHalfTheta,
			    ratioB = Math.sin(slerp*halfTheta) / sinHalfTheta;
			this[0] = (this[0]*ratioA + quat[0]*ratioB);
			this[1] = (this[1]*ratioA + quat[1]*ratioB);
			this[2] = (this[2]*ratioA + quat[2]*ratioB);
			this[3] = (this[3]*ratioA + quat[3]*ratioB);
			return this;
		};
		obj.toString = obj.str = function () {
			return this.toArray().join(',');
		};
		obj.toArray = function () {
			return [
				this[0], this[1],
				this[2], this[3]
			];
		};
		obj.toFloat32Array = function () {
			return new Float32Array(this.toArray());
		};
		obj.toFloat64Array = function () {
			return new Float64Array(this.toArray());
		};
		return obj;
	};
	(module.exports = Quaternion).create(quaternion);
})();